from datetime import datetime
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

import pandas as pd
import urllib.request


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the base
    class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        """
        Parse council provided CSVs to get the latest bin collections for address
        """

        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")
        check_uprn(user_uprn)
        check_postcode(user_postcode)
        # Create Selenium webdriver
        page = (
            f"https://www.leeds.gov.uk/residents/bins-and-recycling/check-your-bin-day"
        )

        driver = create_webdriver(web_driver)
        driver.get(page)

        wait = WebDriverWait(driver, 60)
        postcode_box = wait.until(
            EC.element_to_be_clickable(
                (
                    By.ID,
                    "ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_txtPostCode",
                )
            )
        )
        postcode_box.send_keys(user_postcode)
        postcode_btn_present = wait.until(
            EC.presence_of_element_located(
                (
                    By.ID,
                    "ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_btnSearchAddress",
                )
            )
        )
        postcode_btn = wait.until(
            EC.element_to_be_clickable(
                (
                    By.XPATH,
                    '//*[@id="ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_btnSearchAddress"]',
                )
            )
        )

        postcode_btn.send_keys(Keys.ENTER)

        dropdown_present = wait.until(
            EC.presence_of_element_located(
                (
                    By.XPATH,
                    '//*[@id="ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_ddlAddressList"]/option',
                )
            )
        )
        address_dropdown = wait.until(
            EC.element_to_be_clickable(
                (
                    By.ID,
                    "ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_ddlAddressList",
                )
            )
        )

        dropdown_present.click()

        dropdownSelect = Select(address_dropdown)
        dropdownSelect.select_by_value(str(user_uprn))
        results = wait.until(
            EC.presence_of_element_located(
                (
                    By.ID,
                    "ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_BinResultsDetails",
                )
            )
        )

        data = {"bins": []}  # dictionary for data
        soup = BeautifulSoup(driver.page_source, "html.parser")

        bin_types = soup.find_all("ul", class_="binCollectionTimesList")

        for bin_collection_dates in bin_types:
            bin_collection_list = bin_collection_dates.find_all("li", class_="")

            if bin_collection_list:
                collection_dates = [date.text.strip() for date in bin_collection_list]

                # Convert the collection dates to the desired format
                formatted_dates = [
                    datetime.strptime(date, "%A %d %b %Y").strftime(date_format)
                    for date in collection_dates
                ]

                # Extract the type of bin from the header
                bin_type = bin_collection_dates.find_previous("h3").text.split()[0]

                # Adding data to the 'bins' dictionary for each date
                for date in formatted_dates:
                    dict_data = {"type": bin_type, "collectionDate": date}
                    data["bins"].append(dict_data)

        return data



import json
from collections import OrderedDict
from datetime import datetime
from bs4 import BeautifulSoup
import requests
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
import ssl
import urllib3
import logging


class CustomHttpAdapter(requests.adapters.HTTPAdapter):
    """Transport adapter" that allows us to use custom ssl_context."""

    def __init__(self, ssl_context=None, **kwargs):
        self.ssl_context = ssl_context
        super().__init__(**kwargs)

    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = urllib3.poolmanager.PoolManager(
            num_pools=connections,
            maxsize=maxsize,
            block=block,
            ssl_context=self.ssl_context,
        )


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object

        data = {"bins": []}
        uprn = kwargs.get("uprn")
        web_driver = kwargs.get("web_driver")
        current_month = datetime.today().strftime("%m")
        current_year = datetime.today().strftime("%Y")
        url = (
            f"https://mybins.blackburn.gov.uk/api/mybins/getbincollectiondays?uprn={uprn}&month={current_month}"
            f"&year={current_year}"
        )
        driver = create_webdriver(web_driver)
        driver.get(url)

        soup = BeautifulSoup(driver.page_source, "html.parser")

        # Find the <pre> tag that contains the JSON data
        pre_tag = soup.find("pre")

        if pre_tag:
            # Extract the text content within the <pre> tag

            # Return JSON from response and loop through collections
            json_result = json.loads(pre_tag.contents[0])
            bin_collections = json_result["BinCollectionDays"]
            for collection in bin_collections:
                if collection is not None:
                    bin_type = collection[0].get("BinType")
                    current_collection_date = datetime.strptime(
                        collection[0].get("CollectionDate"), "%Y-%m-%d"
                    )
                    next_collection_date = datetime.strptime(
                        collection[0].get("NextScheduledCollectionDate"), "%Y-%m-%d"
                    )

                    # Work out the most recent collection date to display
                    if (
                        datetime.today().date()
                        <= current_collection_date.date()
                        < next_collection_date.date()
                    ):
                        collection_date = current_collection_date
                    else:
                        collection_date = next_collection_date

                    dict_data = {
                        "type": bin_type,
                        "collectionDate": collection_date.strftime(date_format),
                    }
                    data["bins"].append(dict_data)

                    data["bins"].sort(
                        key=lambda x: datetime.strptime(
                            x.get("collectionDate"), date_format
                        )
                    )

            return data



import re
import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
import time


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64)"}

        uprn = kwargs.get("uprn")
        user_paon = kwargs.get("paon")
        postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")
        driver = create_webdriver(web_driver)
        driver.get(kwargs.get("url"))

        wait = WebDriverWait(driver, 60)
        post_code_search = wait.until(
            EC.presence_of_element_located((By.CLASS_NAME, "form-control"))
        )
        post_code_search.send_keys(postcode)

        submit_btn = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, f"//button[contains(@class, 'mx-name-actionButton3')]")
            )
        )

        submit_btn.send_keys(Keys.ENTER)

        dropdown_options = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, f'//option[contains(text(), "{user_paon}")]')
            )
        )
        parent_element = dropdown_options.find_element(
            By.XPATH, ".."
        )  # Using ".." to move up to the parent element

        # Create a 'Select' for it, then select the first address in the list
        # (Index 0 is "Make a selection from the list")
        dropdownSelect = Select(parent_element)
        dropdownSelect.select_by_visible_text(str(user_paon))

        submit_btn = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, f"//button[contains(@class, 'mx-name-actionButton5')]")
            )
        )

        submit_btn.send_keys(Keys.ENTER)

        results = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, f'//span[contains(@class,"collection-sub")]')
            )
        )

        # Make a BS4 object
        soup = BeautifulSoup(driver.page_source, features="html.parser")
        # Initialize current month and year (you can modify these values based on your requirement)
        data = {"bins": []}

        current_date = datetime.now()

        # Find all elements with class starting with 'mx-name-index-'
        bins = soup.find_all(class_=lambda x: x and x.startswith("mx-name-index-"))

        for bin_item in bins:
            bin_type = bin_item.find(class_="collection-main").text.strip()
            day_of_week_elements = bin_item.find_all(class_="collection-header")
            bin_date = None

            for elem in day_of_week_elements:
                if (
                    elem.text.strip() != bin_type
                ):  # Avoid taking the bin type as the date
                    next_sibling = elem.find_next_sibling()
                    if next_sibling:
                        bin_date_str = next_sibling.text.strip()
                        try:
                            # Try parsing the date string in the format 'dd Month' (e.g., '30 Dec', '5 January')
                            bin_date = datetime.strptime(bin_date_str, "%d %b")
                        except ValueError:
                            try:
                                # If the above format fails, try 'dd MonthName' (e.g., '30 December', '5 January')
                                bin_date = datetime.strptime(bin_date_str, "%d %B")
                            except ValueError:
                                pass

                        if bin_date:
                            # Set the year based on the logic provided
                            if bin_date.month < current_date.month:
                                bin_date = bin_date.replace(year=current_date.year + 1)
                            else:
                                bin_date = bin_date.replace(year=current_date.year)
                            # Format the date to the desired format
                            bin_date = bin_date.strftime("%d/%m/%Y")
                            break
            dict_data = {"type": bin_type, "collectionDate": bin_date}
            data["bins"].append(dict_data)

        return data



import re
import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
import time


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        uprn = kwargs.get("uprn")
        postcode = kwargs.get("postcode")
        full_address = kwargs.get("paon")

        url = "https://my.guildford.gov.uk/customers/s/view-bin-collections"

        web_driver = kwargs.get("web_driver")

        driver = create_webdriver(web_driver)
        driver.get(kwargs.get("url"))

        wait = WebDriverWait(driver, 120)
        post_code_search = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, "//input[contains(@class, 'slds-input')]")
            )
        )

        post_code_search.send_keys(postcode)

        post_code_submit_btn = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, "//button[contains(@class,'slds-button')]")
            )
        )
        post_code_submit_btn.send_keys(Keys.ENTER)

        # Locate the element containing the specified address text
        address_element = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (
                    By.XPATH,
                    f"//lightning-base-formatted-text[contains(text(), '{full_address}')]",
                )
            )
        )

        # Find the associated radio button in the same row (preceding sibling)
        radio_button = address_element.find_element(
            By.XPATH, "../../../../preceding-sibling::td//input[@type='radio']"
        )

        radio_button.send_keys(Keys.SPACE)
        address_submit_btn = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, "//button[contains(@name,'NEXT')]")
            )
        )
        address_submit_btn.send_keys(Keys.ENTER)

        results = wait.until(
            EC.presence_of_element_located((By.CLASS_NAME, "cBinScheduleDisplay"))
        )

        results2 = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, f'//div[contains(@title,"Bin Job")]')
            )
        )
        soup = BeautifulSoup(driver.page_source, features="html.parser")
        # Find all table rows containing bin information
        rows = soup.find_all("tr", class_="slds-hint-parent")

        data = {"bins": []}

        # Extract bin type and next collection date for each row
        for row in rows:
            bin_type = (
                row.find("td", {"data-label": "Bin Job"}).find("strong").text.strip()
                if row.find("td", {"data-label": "Bin Job"})
                else None
            )

            next_collection_date = (
                row.find("td", {"data-label": "Next Collection"}).text.strip()
                if row.find("td", {"data-label": "Next Collection"})
                else None
            )

            if bin_type and next_collection_date:
                # Convert date string to datetime object
                date_format = (
                    "%A, %d %B"  # Adjust the format according to your date string
                )
                try:
                    next_collection_date = datetime.strptime(
                        next_collection_date, date_format
                    )

                    # Logic to determine year
                    current_date = datetime.now()
                    if next_collection_date.month < current_date.month:
                        year = current_date.year + 1
                    else:
                        year = current_date.year

                    # Format the date
                    next_collection_date = next_collection_date.replace(
                        year=year
                    ).strftime("%d/%m/%Y")
                except ValueError:
                    pass

                dict_data = {"type": bin_type, "collectionDate": next_collection_date}
                data["bins"].append(dict_data)
        return data



from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
from selenium.webdriver.common.keys import Keys


# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://selfservice.preston.gov.uk/service/Forms/FindMyNearest.aspx?Service=bins"

        data = {"bins": []}

        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")
        check_paon(user_paon)
        check_postcode(user_postcode)

        # Create Selenium webdriver
        driver = create_webdriver(web_driver)
        driver.get(page)

        # If you bang in the house number (or property name) and postcode in the box it should find your property
        inputElement_address = driver.find_element(
            By.ID,
            "MainContent_txtAddress",
        )

        inputElement_address.send_keys(user_paon)
        inputElement_address.send_keys(" ")
        inputElement_address.send_keys(user_postcode)

        driver.find_element(
            By.ID,
            "btnSearch",
        ).send_keys(Keys.ENTER)

        # Wait for the 'Select your property' dropdown to appear and select the first result
        dropdown = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "MainContent_ddlSearchResults"))
        )
        # Create a 'Select' for it, then select the first address in the list
        # (Index 0 is "Make a selection from the list")
        dropdownSelect = Select(dropdown)
        dropdownSelect.select_by_index(1)

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        # Quit Selenium webdriver to release session
        driver.quit()

        topLevelSpan = soup.find("span", id="MainContent_lblMoreCollectionDates")

        collectionDivs = topLevelSpan.find_all("div", {"id": "container"})

        for collectionDiv in collectionDivs:
            type_and_date_divs = collectionDiv.find_all("b")
            bin_type = type_and_date_divs[0].text

            date_elements = collectionDiv.find_all("li")
            for date_element in date_elements:
                date_string = date_element.find("span").text.split(" ")[1]
                collection_date = datetime.strptime(date_string, "%d/%m/%Y").strftime(
                    date_format
                )

                data["bins"].append(
                    {
                        "type": re.sub(r"[^a-zA-Z0-9,\s]", "", bin_type).strip(),
                        "collectionDate": collection_date,
                    }
                )

        return data



from datetime import datetime
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

import pandas as pd
import urllib.request


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the base
    class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")

        # Create Selenium webdriver
        page = f"https://www.eastriding.gov.uk/environment/bins-rubbish-recycling/bins-and-collections/bin-collection-dates/"

        driver = create_webdriver(web_driver)
        driver.get(page)

        wait = WebDriverWait(driver, 60)

        try:
            accept_cookies = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, '//*[@id="er-cookie-placeholder-settings"]/div/a[1]')
                )
            )

            accept_cookies.click()
        except:
            print(
                "Cookies acceptance element not found or clickable within the specified time."
            )
            pass

        expand_postcode_box = wait.until(
            EC.element_to_be_clickable(
                (By.XPATH, "//a[@href='#when-and-where-should-i-put-my-bin-out']")
            )
        )

        expand_postcode_box.click()

        postcode_box = wait.until(
            EC.element_to_be_clickable(
                (By.XPATH, "//input[@placeholder='Enter your postcode']")
            )
        )
        postcode_box.send_keys(user_postcode)

        postcode_search_btn = wait.until(
            EC.element_to_be_clickable((By.XPATH, "//button[text()='Search']"))
        )
        postcode_search_btn.send_keys(Keys.ENTER)

        dropdown = wait.until(
            EC.presence_of_element_located(
                (
                    By.XPATH,
                    "//div[@class='er-select-wrapper']/select[@class='dropdown']",
                )
            )
        )

        options_present = wait.until(
            EC.presence_of_all_elements_located(
                (By.CSS_SELECTOR, "select.dropdown option")
            )
        )
        drop = Select(dropdown)
        drop.select_by_visible_text(str(user_paon))

        results_present = wait.until(
            EC.presence_of_element_located(
                (
                    By.CLASS_NAME,
                    "results",
                )
            )
        )

        data = {"bins": []}  # dictionary for data
        soup = BeautifulSoup(driver.page_source, "html.parser")

        bin_types = {}  # Dictionary to store bin types

        # Extract bin types from header elements
        header_elements = soup.find_all(
            "li", class_=lambda x: x and x.startswith("header")
        )
        for header_element in header_elements:
            bin_type = header_element.get_text(strip=True)
            bin_class = [cls for cls in header_element.get("class") if cls != "header"]
            if bin_class:
                bin_types[bin_class[0]] = bin_type

        # Extract collection dates and associate them with respective bin types
        date_elements = soup.find_all("li", class_=lambda x: x and x.startswith("date"))
        for date_element in date_elements:
            bin_class = [cls for cls in date_element.get("class") if cls != "date"]
            if bin_class:
                bin_type = bin_types.get(bin_class[0])
                span_text = date_element.find("span").get_text(strip=True)
                collection_date = datetime.strptime(span_text, "%a, %d %b %Y")
                data["bins"].append(
                    {
                        "type": bin_type,
                        "collectionDate": collection_date.strftime(date_format),
                    }
                )

        return data



from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}

        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")
        check_uprn(user_uprn)
        check_postcode(user_postcode)
        # Create Selenium webdriver
        page = (
            f"https://my.nwleics.gov.uk/my-property-finder?address={user_postcode}&go=1"
        )

        driver = create_webdriver(web_driver)
        driver.get(page)

        # If you bang in the house number (or property name) and postcode in the box it should find your property

        # iframe_presense = WebDriverWait(driver, 30).until(
        #    EC.presence_of_element_located((By.ID, "fillform-frame-1"))
        # )

        # driver.switch_to.frame(iframe_presense)
        wait = WebDriverWait(driver, 60)

        address_link = wait.until(
            EC.element_to_be_clickable(
                (By.XPATH, f'//a[contains(@href, "{user_uprn}")]')
            )
        )

        address_link.click()

        refuse_element = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, f'//h3[contains(text(), "Refuse Collection Dates")]')
            )
        )

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        # Find the unordered list containing refuse collection details
        refuse_list = soup.find("ul", class_="refuse")

        current_year = datetime.now().year

        if refuse_list:
            # Iterate through list items within the unordered list
            for li in refuse_list.find_all("li"):
                date = li.find("strong", class_="date").text.strip()  # Extract the date
                waste_type = li.find("a").text.strip()  # Extract the waste type

                # Parse the date from the string
                parsed_date = datetime.strptime(date, "%a %dth %b")
                if parsed_date < datetime(
                    parsed_date.year, parsed_date.month, parsed_date.day
                ):
                    parsed_date = parsed_date.replace(year=current_year + 1)
                else:
                    parsed_date = parsed_date.replace(year=current_year)

                # Append data to your 'bins' list (this replicates your existing logic)
                data["bins"].append(
                    {
                        "type": waste_type,
                        "collectionDate": parsed_date.strftime("%d/%m/%Y"),
                    }
                )
        return data



import re
import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
import time


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64)"}

        uprn = kwargs.get("uprn")
        postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")
        driver = create_webdriver(web_driver)
        driver.get(kwargs.get("url"))

        wait = WebDriverWait(driver, 60)
        post_code_search = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, f"//input[contains(@class, 'searchAddress')]")
            )
        )
        post_code_search.send_keys(postcode)

        submit_btn = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, f"//button[contains(@class, 'searchAddressButton')]")
            )
        )

        submit_btn.send_keys(Keys.ENTER)

        address_link = wait.until(
            EC.presence_of_element_located(
                (By.XPATH, f'//a[contains(@data-uprn,"{uprn}")]')
            )
        )

        address_link.send_keys(Keys.ENTER)

        address_results = wait.until(
            EC.presence_of_element_located(
                (By.CLASS_NAME, "your-collection-schedule-container")
            )
        )

        # Make a BS4 object
        soup = BeautifulSoup(driver.page_source, features="html.parser")
        data = {"bins": []}

        # Get the current month and year
        current_month = datetime.now().month
        current_year = datetime.now().year

        # Function to extract collection data
        def extract_collection_data(collection_div, collection_type):
            if collection_div:
                date_element = (
                    collection_div.find(class_="recycling-collection-day-numeric")
                    or collection_div.find(class_="refuse-collection-day-numeric")
                    or collection_div.find(class_="garden-collection-day-numeric")
                )
                month_element = (
                    collection_div.find(class_="recycling-collection-month")
                    or collection_div.find(class_="refuse-collection-month")
                    or collection_div.find(class_="garden-collection-month")
                )

                if date_element and month_element:
                    collection_date = date_element.get_text(strip=True)
                    collection_month = month_element.get_text(strip=True)

                    # Combine month, date, and year into a string
                    date_string = f"{collection_date} {collection_month} {current_year}"

                    try:
                        # Convert the date string to a datetime object
                        collection_date_obj = datetime.strptime(date_string, "%d %B %Y")

                        # Check if the month is ahead of the current month
                        if collection_date_obj.month >= current_month:
                            # If the month is ahead, use the current year
                            formatted_date = collection_date_obj.strftime(date_format)
                        else:
                            # If the month is before the current month, use the next year
                            formatted_date = collection_date_obj.replace(
                                year=current_year + 1
                            ).strftime(date_format)
                        # Create a dictionary for each collection entry
                        dict_data = {
                            "type": collection_type,
                            "collectionDate": formatted_date,
                        }

                        # Append dictionary data to the 'bins' list in the 'data' dictionary
                        data["bins"].append(dict_data)

                    except ValueError as e:
                        # Handle the case where the date format is invalid
                        formatted_date = "Invalid Date Format"

        # Extract Recycling collection data
        extract_collection_data(
            soup.find(class_="container-fluid RegularCollectionDay").find_next_sibling(
                "div"
            ),
            "Recycling",
        )

        # Extract Refuse collection data
        for refuse_div in soup.find_all(class_="container-fluid RegularCollectionDay"):
            extract_collection_data(refuse_div, "Refuse")

        # Extract Garden Waste collection data
        extract_collection_data(
            soup.find(class_="container-fluid gardenwasteCollectionDay"), "Garden Waste"
        )

        # Print the extracted data
        return data
